#include <stdio.h>
char buf[1256];
char last[1256];
char *p;

main ()
{
  while (gets (buf) != (char *) NULL)
    {
	for(p=buf;*p !='#';p++);
	*p = '\0';
        if(strcmp(last,buf) != 0){ printf("#%s\n%s\n",p+1,buf); strcpy(last,buf);}

    }
}
